@@ -165,11 +165,9 @@ class hotspot_backend:
                 ] 
             ) 
         )
-        # TODO: Listen for the request status signal and raise error if request fails
-        # TODO: Use the signal listerner and mainloop for the wait here
+        # TODO: Make a (separate) DBUS listener to wait untill the selected connection is up
         time.sleep(5)
 
-
     def icd2_state_listener(self, *args):
         print "icd2_state_listener got args:\n";
         print args
@@ -218,22 +216,35 @@ class hotspot_backend:
         """Stores the current connection (if any) and opens the selected one"""
         self.get_icd2_state()
         # Request the GPRS connection
+        try:
+            if (    self.connection == self.state['icd']['old_connection_name']
+                and self.state['icd']['old_connection_type'] == 'GPRS'):
+                    # Previous connection is same as the one we want, save some time and cpu-cycles
+                    return
+        except KeyError, e:
+            pass
         self.icd2_open_connection(self.connection, 'GPRS')
 
     def start_configure_wlan(self):
         """Configures the WLAN: set scan interval to never then raise the interface"""
         # Disable WLAN scan (old interval was stored to state earlier)
         self.gc.set_int('/system/osso/connectivity/network_type/search_interval', 0) 
-        # PONDER: Create temporary IAP and use IcD2 instead ???
+        # PONDER: Create temporary IAP and use IcD2 instead ??? (<- it might be impossible to use two IAPs at the same time)
         helpers.exec_cmd('ifconfig ' + self.wlanif + ' down')
+        # TODO: Verify that the interface is fully down so we can configure it, now we just sleep a moment
+        time.sleep(3)
         helpers.exec_cmd('iwconfig ' + self.wlanif + ' mode ad-hoc')
         helpers.exec_cmd('ifconfig ' + self.wlanif + ' up')
+        # TODO: Verify that the interface is fully up so we can configure it, now we just sleep a moment
+        time.sleep(3)
         if (    self.encryption_algo
             and self.encryption_key):
             helpers.exec_cmd('iwconfig ' + self.wlanif + ' key ' + helpers.hex_encode(helpers.verify_wep_key(self.encryption_key)) + ' restricted')
             #helpers.exec_cmd('iwconfig ' + self.wlanif + ' key ' + keyhexenc)
         helpers.exec_cmd('iwconfig ' + self.wlanif + ' essid "' + self.ssid + '"')
         helpers.exec_cmd('ifconfig ' + self.wlanif + ' ' + self.network + '1 netmask 255.255.255.0 up')
+        # TODO: Verify that the interface is fully up, now we just sleep a moment
+        time.sleep(3)
 
     def start_run_dnsmasq(self):
         """Starts the dnsmasq daemon with dhcp enabled"""
@@ -328,9 +339,12 @@ class hotspot_backend:
         """Takes down interface"""
         # It seems we need to take the wlan interface back to managed mode or there will be trouble
         helpers.exec_cmd('ifconfig ' + self.wlanif + ' down')
+        # TODO: Verify that device is actually configured and properly down, for now we just sleep a moment
+        time.sleep(3)
         helpers.exec_cmd('iwconfig ' + self.wlanif + ' mode managed')
         helpers.exec_cmd('iwconfig ' + self.wlanif + ' key open')
-        time.sleep(1)
+        # TODO: Verify that device is actually configured and properly down, for now we just sleep a moment
+        time.sleep(3)
 
     def stop_close_connection(self):
         """Reopens the old connection if there was any, then restores WLAN scan interval"""
